      implicit real*8 (a-h,o-z)

      include 'CONSTCOM.'
      INCLUDE 'APSCOM5.'
      INCLUDE 'PDFCOM.'
      include 'PRECCOM.'

      real*8 xbeic(20), q2eic(12)
      data xbeic /    1.2925D-04, 2.0484D-04, 3.2465D-04, 5.1453D-04
     ,   ,8.1548D-04, 1.2924D-03, 2.0484D-03, 3.2465D-03, 5.1453D-03
     ,   ,8.1548D-03, 1.2925D-02, 2.0484D-02, 3.2465D-02, 5.1453D-02
     ,   ,8.1548D-02, 1.2924D-01, 2.0484D-01, 3.2465D-01, 5.1453D-01
     ,   ,8.1548D-01/
      data q2eic / 1.3891E+00, 2.4703E+00, 4.3928E+00, 7.8117E+00
     ,            ,1.3891E+01, 2.4703E+01, 4.3928E+01, 7.8117E+01
     ,            ,1.3891E+02, 2.4703E+02, 4.3928E+02, 7.8117E+02 /

!  Set up constants, grid spacing, etc.
      call initgridconst
!  Set improved integration accuracy for the benchmarking,
!  for the most of practical applications nf...=1 is sufficient   
      nmthq=8       ! heavy-quark PDF generation 
      nf2hq=8       ! heavy-quark contribution to F_2,L 
      nf2qcd1=8     ! light-parton contribution to F_2 (x<0.1) 
      nf2qcd2=8     ! light-parton contribution to F_2 (x>0.1) 
      nflqcd=8      ! light-parton contribution to F_L 

!  The NNLO approximation for the strong coupling 
      kordalps=2

!  Set up the 3-flavour NNLO PDFs and fill the 3-flavour PDF grid
!  including the strong coupling values
      kschemepdf=0
      kordpdf=2
      call pdffillgrid

!  Filling the 4-flavour PDF grid  
      call fillvfngrid

!  Set the factorization scale to Q, as motivated by the VFN schemes
      hqscale2=0d0

! The non-singlet contribution is included into the 3-flavor term
      hqnons=.true.

      print *,'The c-quark semi-inclusive structure functions F_2^c', 
     -' (first column: momentum transferred in GeV^2,',
     -' second column: x, third column: 3-flavour FFNS value of F_2^c',
     -' 4-th column: 3-flavour FFNS value of F_L^c',
     -' 5-th column: asymptotic value of F_2^c in the 3-flavour scheme,',
     -' 6-th column: ZMVFNS value of F_2^c,',
     -' 7-th column: GMVFNS value of F_2^c in the BMSN prescription)'

!  Check the case of c-quark production (ihqn=10 for the case of b-quark)
      ihqn=8

      print *,'O(\alpha_s^2) approximation'
      kordhq=1  !   O(\alpha_s^2)
      do n=1,12
        q2=q2eic(n)
        do nn=1,20
          xb=xbeic(nn)
          a1=f2charm_ffn(xb,q2,ihqn)
          a5=flcharm_ffn(xb,q2,ihqn)
          a2=f2h_asymp(3,1,22,xb,q2,ihqn)
          a3=f2h_vfn(3,1,22,xb,q2,ihqn)
          a4=f2h_bmsn(3,1,22,xb,q2,ihqn)
          write (*,fmt='(9e13.4)') q2,xb,a1,a5,a2,a3,a4
        end do
      end do

      print *,'O(\alpha_s^3) approximation'
      kordhq=2  !   O(\alpha_s^3)
      do n=1,12
        q2=q2eic(n)
        do nn=1,20
          xb=xbeic(nn)
          a1=f2charm_ffn(xb,q2,ihqn)
          a5=flcharm_ffn(xb,q2,ihqn)
          write (*,fmt='(6e13.4)') q2,xb,a1,a5
        end do
      end do

      print *,'The NNLO inclusive proton structure functions F_2,L^incl', 
     -' (first column: momentum transferred in GeV^2,',
     -' second column: x, third column: the value of F_2^incl', 
     -' with F_2^c,b in the approximate NNLO FFNS, 4-th column:', 
     -' value of F_L^incl with NNLO F_L^c,b in the approximate', 
     -' NNLO FFNS, 5-th column: value of F_2^incl with', 
     -' F_2^c in the NLO BMSN prescription'

      do n=1,12
        q2=q2eic(n)
        do nn=1,20
          xb=xbeic(nn)
          kordhq=2                ! O(\alpha_s^3) for the heavy quark production
          a1=f2charm_ffn(xb,q2,8)
          a2=flcharm_ffn(xb,q2,8)
          a3=f2charm_ffn(xb,q2,10)
          a4=flcharm_ffn(xb,q2,10)
          kordhq=1                ! O(\alpha_s^2) for the heavy quark production
          a5=f2h_bmsn(3,1,22,xb,q2,8)

          kordf2=2      ! O(\alpha_s^2) for the light parton F_2
          b1=f2qcd(3,1,22,xb,q2)
          kordfl=2      ! O(\alpha_s^2) for the light parton F_L
          b2=flqcd(3,1,22,xb,q2)

          write (*,fmt='(10e13.4)') q2,xb,a1+a3+b1,a2+a4+b2,a5+a3+b1
        end do
      end do

      stop
      end
