      implicit real*8 (a-h,o-z)

      INCLUDE 'CONSTCOM.'
      INCLUDE 'APSCOM5.'
      INCLUDE 'PDFCOM.'

      real*8 termplus(10,0:50,100),termtotplus(10,0:50,100)
     ,      ,termdisp(10,100)
      npartot=0
!  Set up constants, grid spacing, etc.
      call initgridconst

!  Set up the 5-flavour NNLO PDFs
      kschemepdf=2
      kordpdf=1

!  Set the collision energy and the W mass
      scms=7d3
      rmuf=80.398

      tau=rmuf/scms

      do kpdfset=0,npartot
!  Fill the PDF grid for the set #kpdfset
        call pdffillgrid

!  Loop over rapidity
        do i=1,40
          yrap=(i-1)*0.1d0
          xx1=tau*exp(yrap)
          xx2=tau*exp(-yrap)

!  Fill the luminosity array 
          do klum=1,5
            termplus(klum,kpdfset,i)=pdflum(rmuf**2,xx1,xx2,klum)
          end do
        end do
      end do

      print *,' The differential W+ production luminosity', 
     -' in the proton-proton collisions at the s.c.m. energy of 7 TeV', 
     -' (first column: the W+ rapidity,',
     -' second column: u-dbar contribution with the uncertainty,',
     -' third column: u-sbar contribution with the uncertainty,',
     -' fourth column: c-sbar contribution with the uncertainty).'

      do i=1,40
        yrap=(i-1)*0.1d0
        xx1=tau*exp(yrap)
        xx2=tau*exp(-yrap)

        do klum=1,5
          termdisp(klum,i)=0.

          do kpdfset=1,npartot          
            termdisp(klum,i)=termdisp(klum,i)
     +    +(termplus(klum,kpdfset,i)-termplus(klum,0,i))**2
          enddo  
!  In the case of asymmetric PDF errors 
!  take average of the positive and negative ones  
#ifdef ABKM09
          termdisp(klum,i)=sqrt(termdisp(klum,i))
#else
          termdisp(klum,i)=sqrt(termdisp(klum,i)/2.)
#endif

        end do

        write (*,fmt='(e13.4,3(e13.4,a2,e10.4))') yrap
     ,        ,termplus(1,0,i),'+-',termdisp(1,i)
     ,        ,termplus(3,0,i),'+-',termdisp(3,i)
     ,        ,termplus(5,0,i),'+-',termdisp(5,i)
      end do

      stop
      end
c---------------------
      real*8 function pdflum(q2,xx1,xx2,klum)
      implicit real*8(a-h,o-z)

      INCLUDE 'PDFCOM.'

      up1=xqg(4,xx1,q2,kschemepdf)
      dn1=xqg(2,xx1,q2,kschemepdf)
      usea1=xqg(5,xx1,q2,kschemepdf)
      dsea1=xqg(3,xx1,q2,kschemepdf)
      str1=xqg(6,xx1,q2,kschemepdf)
      chm1=xqg(8,xx1,q2,kschemepdf)
      bot1=xqg(10,xx1,q2,kschemepdf)
      glu1=xqg(1,xx1,q2,kschemepdf)

      up2=xqg(4,xx2,q2,kschemepdf)
      dn2=xqg(2,xx2,q2,kschemepdf)
      usea2=xqg(5,xx2,q2,kschemepdf)
      dsea2=xqg(3,xx2,q2,kschemepdf)
      str2=xqg(6,xx2,q2,kschemepdf)
      chm2=xqg(8,xx2,q2,kschemepdf)
      bot2=xqg(10,xx2,q2,kschemepdf)
      glu2=xqg(1,xx2,q2,kschemepdf)

      sumqqflux=0.

c W+ production u-dbar (p-p collisions)
      if (klum.eq.1) then 
        sumqqflux=(up1*dsea2 + up2*dsea1)*0.9474 
      end if

c W+ production u-dbar (p-pbar collisions)
      if (klum.eq.2) then 
        sumqqflux=(up1*dn2 + usea2*dsea1)*0.9474 
      end if

c W+ production u-sbar (p-p collisions)
      if (klum.eq.3) then 
        sumqqflux=(up1*str2 + up2*str1)*0.0509D0
      end if

c W+ production u-sbar (p-pbar collisions)
      if (klum.eq.4) then 
        sumqqflux=(up1*str2 + usea2*str1)*0.0509D0
      end if

c W+ production c-sbar (pp or p-pbar collisions)
      if (klum.eq.5) then 
        sumqqflux=(chm1*str2 + chm2*str1)*0.9474 
      end if

      pdflum=sumqqflux

      return 
      end
